package com.supermap.imobile.bufferanalyst;

import com.supermap.analyst.BufferAnalystGeometry;
import com.supermap.analyst.BufferAnalystParameter;
import com.supermap.analyst.BufferEndType;
import com.supermap.data.Color;
import com.supermap.data.CursorType;
import com.supermap.data.DatasetType;
import com.supermap.data.DatasetVector;
import com.supermap.data.DatasetVectorInfo;
import com.supermap.data.Datasets;
import com.supermap.data.Environment;
import com.supermap.data.GeoRegion;
import com.supermap.data.GeoStyle;
import com.supermap.data.Geometry;
import com.supermap.data.PrjCoordSys;
import com.supermap.data.Recordset;
import com.supermap.data.Workspace;
import com.supermap.data.WorkspaceConnectionInfo;
import com.supermap.data.WorkspaceType;
import com.supermap.mapping.Action;
import com.supermap.mapping.Layer;
import com.supermap.mapping.MapControl;
import com.supermap.mapping.TrackingLayer;
import android.app.Activity;
import android.os.Bundle;
import android.util.DisplayMetrics;
import android.view.Display;
import android.view.MotionEvent;
import android.view.View;
import android.view.Window;
import android.view.WindowManager;
import android.view.View.OnClickListener;
import android.view.View.OnTouchListener;
import android.widget.Button;
import android.widget.EditText;
import android.widget.ImageButton;
import android.widget.Toast;
import android.widget.ZoomControls;

/**
 * <p>
 * Title:Buffer Analysis
 * </p>
 * 
 * <p>
 * Description:
 * ============================================================================>
 * ------------------------------Statement----------------------------
 * This file is SuperMap iMobile for Android SampleCode 
 * Copyright: SuperMap Software Co., Ltd.
 * ----------------------------------------------------------------
 * ---------------------SuperMap iMobile for Android SampleCode------------------------
 * 
 * 1. Introduction: How to query data in the buffer area, and display them on the MapControl
 * 2. Data:/sdcard/SampleData/City/Changchun.smwu
 * 3. Members: 
 *      QueryParameter.setSpatialQueryObject Method
 *      QueryParameter.setSpatialQueryMode Method
 *      SpatialQueryMode.CONTAIN Constant
 *      SpatialQueryMode.INTERSECT Constant
 *      SpatialQueryMode.DISJOINT Constant
 *      Map.findSelection Method
 *      Selecttion.toRecordset Method
 *      Selecttion.fromRecordset Method
 *      DatasetVector.query Method
 * 4. Steps:
 *   (1) Input the buffer radius(number)
 *   (2) Click Select. Long press the object which you want to create buffer area
 *   (3) The query result will be shown on the map
 * ------------------------------------------------------------------------------
 * ============================================================================>
 * </p> 
 * 
 * <p>
 * Company: SuperMap Software Co., Ltd.
 * </p>
 * 
 */

public class MainFrame extends Activity implements OnTouchListener{
    /** Called when the activity is first created. */
	
	private MapControl m_mapControl; // MapControl	
	private Workspace m_woWorkspace; // Workspace
	
	private ZoomControls m_Zoom;
	private ImageButton m_btnSelect;
	private ImageButton m_btnPan;
	private Button m_btnQuery;
	private EditText m_edtDistance;
	String sdcard = android.os.Environment.getExternalStorageDirectory().getAbsolutePath().toString();
	
    @Override
    public void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        // Set path
        Environment.setLicensePath(sdcard+"/SuperMap/license/");
        Environment.initialization(this);
        
        // Hide the titlebar
        requestWindowFeature(Window.FEATURE_NO_TITLE);
        // Hide the taskbar
        getWindow().setFlags(WindowManager.LayoutParams.FLAG_FULLSCREEN,
        		WindowManager.LayoutParams.FLAG_FULLSCREEN);
        
        setContentView(R.layout.main);
        openMap();       

        initView();
    }
    
 // Open the map
    private boolean openMap(){
    	
    	// Get the parameters of the current screen
        final Display display = getWindowManager().getDefaultDisplay(); 
        DisplayMetrics dm = new DisplayMetrics();
        display.getMetrics(dm);

        if(openWorkspace()){ 
            // Connect the MapControl to the workspace    
        	m_mapControl = (MapControl)findViewById(R.id.map_control);
      
        	m_mapControl.getMap().setWorkspace(m_woWorkspace);      	
        	m_mapControl.getMap().setMapDPI(dm.densityDpi);
            m_mapControl.setOnTouchListener(this);

            // Open the first map 
            String mapName = m_woWorkspace.getMaps().get(0);
            boolean isOpenMap = m_mapControl.getMap().open(mapName);
            if(isOpenMap){
                // Refresh the map
                m_mapControl.getMap().refresh();
                
                Layer roadLayer = m_mapControl.getMap().getLayers().get("RoadLine1@changchun");
                if(roadLayer == null)
                	return false;
                else
                	roadLayer.setSelectable(true);
                
                //Set other layers uneditable
                for(int i = 0; i < m_mapControl.getMap().getLayers().getCount(); i++)
                {
                	Layer tempLayer = m_mapControl.getMap().getLayers().get(i);
                	if(roadLayer != tempLayer)
                	{
                		tempLayer.setSelectable(false);
                	}
                }
            }  
            return true;
        }
        return false;
    }
        
    // Open /sdcard/SampleData/City/Changchun.smwu
    private boolean openWorkspace(){
        m_woWorkspace = new Workspace();

        WorkspaceConnectionInfo m_info = new WorkspaceConnectionInfo();
        m_info.setServer(sdcard+"/SampleData/City/Changchun.smwu");
        m_info.setType(WorkspaceType.SMWU);
        
        return m_woWorkspace.open(m_info);
    } 
    
 // Initializing the control, binding event-listener
	private void initView(){
		
		m_Zoom = (ZoomControls)findViewById(R.id.zoomControls1);
		m_Zoom.setOnZoomOutClickListener(new OnClickListener() {
			
			public void onClick(View v) {
				// TODO Auto-generated method stub
				m_mapControl.getMap().zoom(0.5);
				m_mapControl.getMap().refresh();
			}
		});
		m_Zoom.setOnZoomInClickListener(new OnClickListener() {
			
			public void onClick(View v) {
				// TODO Auto-generated method stub
				m_mapControl.getMap().zoom(2);
				m_mapControl.getMap().refresh();
			}
		});
		
		m_btnSelect = (ImageButton)findViewById(R.id.btn_selectGeo);
		m_btnSelect.setOnClickListener(new selectBtnlistener());
		
		m_btnPan = (ImageButton)findViewById(R.id.btn_pan);
		m_btnPan.setOnClickListener(new panBtnlistener());
		
		m_btnQuery = (Button)findViewById(R.id.btn_searchGeo);
		m_btnQuery.setOnClickListener(new queryBtnlistener());
		
		m_edtDistance = (EditText)findViewById(R.id.edt_range);
	}

	// Buffer query
	class queryBtnlistener implements OnClickListener{

		public void onClick(View v) {
			// TODO Auto-generated method stub
			
			Recordset queryRecordset = null;
			Layer ly = m_mapControl.getMap().getLayers().get("RoadLine1@changchun");
			if(ly.getSelection() != null)
			{
				queryRecordset = ly.getSelection().toRecordset();
			}
            
			TrackingLayer tLayer = m_mapControl.getMap().getTrackingLayer();
			tLayer.clear();
			GeoRegion geometryBuffer;
            
			if (queryRecordset != null && queryRecordset.getRecordCount() != 0) {
				// Create the buffer area for the object, and save the result to the dataset
				Datasets datasets = m_woWorkspace.getDatasources().get(0).getDatasets();
				String dtName = datasets.getAvailableDatasetName("da");
				DatasetVectorInfo dd = new DatasetVectorInfo(dtName, DatasetType.REGION);
				DatasetVector d = datasets.create(dd);
				Recordset rr = d.getRecordset(false, CursorType.DYNAMIC);
		        while (!queryRecordset.isEOF()) {
		        	// Set the parameters of buffer analysis
		    	    BufferAnalystParameter bufferAnalystParam = new BufferAnalystParameter();
		    	    bufferAnalystParam.setEndType(BufferEndType.ROUND);
		    	    int distance = 5;
		    	    try{
		    	    	distance = Integer.valueOf(m_edtDistance.getText().toString());
		    	    }catch (Exception e) {
		    	    	distance = 5;
					}
		    	    distance = distance<=0?5:distance;
		    	    bufferAnalystParam.setLeftDistance(distance);
		    	    bufferAnalystParam.setRightDistance(distance);
		    	    
		            Geometry geoForBuffer = queryRecordset.getGeometry();
		            PrjCoordSys prj = queryRecordset.getDataset().getPrjCoordSys();
		            geometryBuffer = BufferAnalystGeometry.createBuffer(geoForBuffer, bufferAnalystParam, prj);
		            
		            // Set styles
			        GeoStyle style = new GeoStyle();
			        style.setLineColor((new Color(50,244,50)));
			        style.setLineSymbolID(0);
			        style.setLineWidth(0.5);
			        style.setMarkerSymbolID(351);
			        style.setMarkerSize(new com.supermap.data.Size2D(5,5));
			        style.setFillForeColor(new Color(244,50,50));
			        style.setFillOpaqueRate(70);
		            
		            geometryBuffer.setStyle(style);
		            
		            tLayer.add(geometryBuffer, "");
		            queryRecordset.moveNext();
		            
					rr.addNew((Geometry)geometryBuffer);
					rr.update();
		        }
				rr.dispose();
			} else {
				Toast.makeText(MainFrame.this, "There is no object selected." , Toast.LENGTH_SHORT).show();
			}

			
	        m_mapControl.getMap().refresh();
			
	        // Release resources
	        queryRecordset.dispose();	
		}
	}
	
	// Button
	class selectBtnlistener implements OnClickListener{

		public void onClick(View v) {
			// TODO Auto-generated method stub
			m_mapControl.setAction(Action.SELECT);
		}
	}
	
	// Pan
	class panBtnlistener implements OnClickListener{
		
		public void onClick(View v) {
			// TODO Auto-generated method stub
			m_mapControl.setAction(Action.PAN);
		}
	}
	
	public boolean onTouch(View v, MotionEvent event) {
		m_mapControl.onMultiTouch(event);
		return true;
	}
}